package com.persagy.server.syn.controllers.obj

import com.persagy.base.extensions.toJson
import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.models.entities.dictnew.DictProject
import com.persagy.server.datacenter.models.entities.dictnew.Group
import com.persagy.server.datacenter.models.entities.objects.Building
import com.persagy.server.datacenter.models.entities.objects.Equipment
import com.persagy.server.datacenter.models.entities.objects.Floor
import com.persagy.server.datacenter.models.entities.objects.Project
import com.persagy.server.syn.models.api.obj.BuildingApi
import com.persagy.server.syn.models.api.obj.EquipApi
import com.persagy.server.syn.models.api.obj.ProjectApi
import com.persagy.server.syn.models.tools.BuildingUtil
import com.persagy.server.syn.models.tools.EquipUtil
import com.persagy.server.syn.models.tools.FloorUtil
import com.persagy.server.syn.models.tools.ProjectUtil
import com.persagy.server.syn.services.dict.DClassDefService
import com.persagy.service.SBaseService
import com.persagy.service.SObjectService
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.responses.SBaseResponse
import io.swagger.annotations.Api
import io.swagger.annotations.ApiOperation
import org.slf4j.LoggerFactory
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

/**
 * 对象，中台到数据中心
 *
 * @author wx  <zhangweixin@sagacloud.com>
 * @date  2020/10/18 10:06
 */
@Api(tags = ["0003、对象-中台到数据中心"])
@RestController
@RequestMapping("/project/sync")
class ObjectFromToControllers {
    companion object {
        // 日志
        private val logger = LoggerFactory.getLogger(ObjectFromToControllers::class.java)
    } // Companion object

    /** 集团服务 */
    private val groupService = SBaseService(SMybatisDao(Group::class.java))
    /** 字典项目服务 */
    private val dictProjectService = SBaseService(SMybatisDao(DictProject::class.java))
    /** 数据中心项目服务 */
    private val projectService = SObjectService(SMybatisDao(Project::class.java))
    /** 数据中心建筑服务 */
    private val buildingService = SObjectService(SMybatisDao(Building::class.java))
    /** 数据中心楼层服务 */
    private val floorService = SObjectService(SMybatisDao(Floor::class.java))
    /** 数据中心设备服务 */
    private val equipmentService = SObjectService(SMybatisDao(Equipment::class.java))

    /**
     * 项目信息 - 数据中台存储到数据中心，根据集团获取集团写所有项目
     */
    @ApiOperation(value = "项目信息-数据中台存储到数据中心，根据集团获取集团写所有项目 ", notes = "")
    @PostMapping(value = ["/create"])
    fun projectGroupAll(): SBaseResponse {
        return try {
            /** 集团数据 */
            DClassDefService.groupQuerys()
            /** 项目数据 */
            DClassDefService.dictProject()

            val groupList = dictProjectService.selectAll().exec()
            for (group in groupList) {
                val projectQueryList = ProjectApi.projectQuery(group.id!!)

                if (projectQueryList.size > 0) {
                    for (projectQuery in projectQueryList) {
                        val projectDataConversion = ProjectUtil.projectDataConversion(group.groupCode!!, projectQuery)
                        logger.debug("项目：  ${projectDataConversion.toJson()} ")
                        projectService.replace(projectDataConversion)
                    }
                }
            }

            SBaseResponse(SResponseType.success)
        } catch (e: Exception) {
            e.printStackTrace()
            SBaseResponse(SResponseType.failure, e.message!!)
        }

    }

    /**
     * 建筑信息 - 数据中台存储到数据中心
     */
    @ApiOperation(value = "建筑信息-数据中台存储到数据中心 ", notes = "switch true  根据输入的项目 id  同步建筑，false 同步所有项目的 建筑")
    @PostMapping(value = ["/building-syn"])
    fun buildingSyn(projectId: String, switch: Boolean = true): SBaseResponse {
        return try {
            if (switch) {
                val dictProject = dictProjectService.select(SFilter.eq("id", projectId)).entity()
                val buildingSynList = BuildingApi.buildingSyn(projectId,"building")
                if (buildingSynList.size > 0) {
                    for (buildingSyn in buildingSynList) {
                        val buildingDataConversion =
                            BuildingUtil.buildingDataConversion(projectId, dictProject!!.groupCode!!, buildingSyn)
                        logger.debug("建筑：  ${buildingDataConversion.toJson()} ")
//                        buildingService.replace(buildingDataConversion)

                    }
                }

            } else {
                /** 查询所有的项目 */
                val projectList = projectService.selectAll().exec()
                for (project in projectList){
                    val dictProject = dictProjectService.select(SFilter.eq("id", project.id!!)).entity()
                    val buildingSynList = BuildingApi.buildingSyn(projectId,"building")
                    if (buildingSynList.size > 0) {
                        for (buildingSyn in buildingSynList) {
                            val buildingDataConversion =
                                BuildingUtil.buildingDataConversion(project.id!!, dictProject!!.groupCode!!, buildingSyn)
                            logger.debug("建筑：  ${buildingDataConversion.toJson()} ")
                            buildingService.replace(buildingDataConversion)
                        }
                    }
                }
            }
            SBaseResponse(SResponseType.success)
        } catch (e: Exception) {
            e.printStackTrace()
            SBaseResponse(SResponseType.failure, e.message!!)
        }

    }


    /**
     * 楼层 - 中台楼层同步到数据中心
     */
    @ApiOperation(value = "楼层信息-数据中台存储到数据中心 ", notes = "switch true  根据输入的项目 id  同步楼层，false 同步所有项目的 楼层")
    @PostMapping(value = ["/floor-syn"])
    fun floorSyn(projectId: String,switch: Boolean = true): SBaseResponse {
        return try {
            if (switch){
                val dictProject = dictProjectService.select(SFilter.eq("id", projectId)).entity()
                val floorSynList = BuildingApi.buildingSyn(projectId, "floor")
                if (floorSynList.size>0) {
                    for (floorSyn in floorSynList){
                        val floor = FloorUtil.floorSyn(projectId, dictProject!!.groupCode!!, floorSyn)
                        floorService.replace(floor)
                    }
                }
            } else {
                /** 查询所有的项目 */
                val projectList = projectService.selectAll().exec()
                for (project in projectList){
                    val dictProject = dictProjectService.select(SFilter.eq("id", project.id!!)).entity()
                    val floorSynList = BuildingApi.buildingSyn(projectId, "floor")
                    if (floorSynList.size>0) {
                        for (floorSyn in floorSynList){
                            val floor = FloorUtil.floorSyn(project.id!!, dictProject!!.groupCode!!, floorSyn)
                            floorService.replace(floor)
                        }
                    }
                }
            }
            SBaseResponse(SResponseType.success)
        } catch (e: Exception) {
            e.printStackTrace()
            SBaseResponse(SResponseType.failure, e.message!!)
        }
    }


    /**
     * 设备
     */
    @ApiOperation(value = "设备信息-数据中台存储到数据中心 ", notes = "switch true  根据输入的项目 id  同步楼层，false 同步所有项目的 楼层")
    @PostMapping(value = ["/equip-syn"])
    fun equipSyn(projectId: String,switch: Boolean = true,isBuildingFloor:Boolean = true,page: Int = 1): SBaseResponse {
        return try {
            var num = page
            if (switch) {
                val dictProject = dictProjectService.select(SFilter.eq("id", projectId)).entity()
                val equipSynApi = EquipApi.equipSynApi(projectId, page)
                val equipDataList = equipSynApi.data
                for (equipData in equipDataList!!){
                    val equipSyn = EquipUtil.equipSyn(projectId, dictProject!!.groupCode!!, equipData)
                    if (isBuildingFloor){
                        /** 查询建筑 */
                        /** 查询楼层 */

                    }
                    equipmentService.replace(equipSyn)
                }
                if (equipSynApi.data!!.size>= 1000){
                    num ++
                    equipSyn(projectId,switch,isBuildingFloor,num)
                }
            } else {
                val projectList = projectService.selectAll().exec()
                for (project in projectList){

                    val dictProject = dictProjectService.select(SFilter.eq("id", project.id!!)).entity()
                    val equipSynApi = EquipApi.equipSynApi(project.id!!, page)
                    val equipDataList = equipSynApi.data
                    for (equipData in equipDataList!!){
                        val equipSyn = EquipUtil.equipSyn(projectId, dictProject!!.groupCode!!, equipData)
                        if (isBuildingFloor){
                            /** 查询建筑 */
                            /** 查询楼层 */

                        }
                        equipmentService.replace(equipSyn)
                    }
                    if (equipSynApi.data!!.size>= 1000){
                        num ++
                        equipSyn(projectId,switch,isBuildingFloor,num)
                    }
                }
            }
            SBaseResponse(SResponseType.success)
        } catch (e: Exception) {
            e.printStackTrace()
            SBaseResponse(SResponseType.failure, e.message!!)
        }
    }

}